@@ -2,10 +2,12 @@
  *
  * File:    Fwknop.java
  *
- * Author:  Max Kastanas <max2idea@users.sourceforge.net>
- *
  * Purpose: A JNI wrapper for Damien Stuart's implementation of fwknop client
  *
+ *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
+ *
  *  License (GNU General Public License):
  *
  *     This program is distributed in the hope that it will be useful,
@@ -96,8 +98,9 @@ public class Fwknop extends Activity {
     private EditText mPasswd;
     private EditText mHmac;
     private EditText mDestip;
-    private Spinner mAccessProto;
-    private EditText mAccessPort;
+    private EditText mDestport;
+    private EditText mTCPAccessPorts;
+    private EditText mUDPAccessPorts;
     private EditText mFwTimeout;
     private ImageButton mUnlock;
     private String access_str;
@@ -105,6 +108,7 @@ public class Fwknop extends Activity {
     private String passwd_str;
     private String hmac_str;
     private String destip_str;
+    private String destport_str;
     private String fw_timeout_str;
     private CheckBox mCheck;
     private String externalIP = "";
@@ -234,7 +238,7 @@ public class Fwknop extends Activity {
         }
     }
 
-//    Another Generic Messanger
+//    Another Generic Messenger
     public static void sendHandlerMessage(Handler handler, int message_type) {
         Message msg1 = handler.obtainMessage();
         Bundle b = new Bundle();
@@ -262,28 +266,48 @@ public class Fwknop extends Activity {
         SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
         SharedPreferences.Editor edit = prefs.edit();
 //
-        if (this.mAccessProto != null
-                && (this.mAccessProto.getSelectedItem().toString().trim().toLowerCase().equals("tcp")
-                || this.mAccessProto.getSelectedItem().toString().trim().toLowerCase().equals("udp"))) {
-            this.access_str = mAccessProto.getSelectedItem().toString().toLowerCase() + "";
-            edit.putString("accessProto_str", mAccessProto.getSelectedItem().toString());
-        } else {
-            this.UIAlert("Input error", "Please enter a valip protocol (tcp/udp)", this);
+        this.access_str = "";
+        if (this.mTCPAccessPorts != null) {
+            if(!this.mTCPAccessPorts.getText().toString().equals("")){
+                String[] ports = this.mTCPAccessPorts.getText().toString().split(",");
+                for(int i = 0; i < ports.length; i++){
+                    try {
+                        int port = Integer.parseInt(ports[i]);
+                        if(i > 0)
+                            this.access_str = this.access_str + ",";
+                        this.access_str = this.access_str + "tcp/" + port;
+                    } catch (Exception e) {
+                        this.UIAlert("Input error", ports[i] + " is not a valid port number", this);
+                        return;
+                    }
+                }
+            }
+            edit.putString("tcpAccessPorts_str", mTCPAccessPorts.getText().toString());
+        }
+
+        if (this.mUDPAccessPorts != null) {
+            if(!this.mUDPAccessPorts.getText().toString().equals("")){
+                String[] ports = this.mUDPAccessPorts.getText().toString().split(",");
+                for(int i = 0; i < ports.length; i++){
+                    try {
+                        int port = Integer.parseInt(ports[i]);
+                        if(this.access_str != null && !this.access_str.equals(""))
+                            this.access_str = this.access_str + ",";
+                        this.access_str = this.access_str + "udp/" + port;
+                    } catch (Exception e) {
+                        this.UIAlert("Input error", ports[i] + " is not a valid port number", this);
+                        return;
+                    }
+                }
+            }
+            edit.putString("udpAccessPorts_str", mUDPAccessPorts.getText().toString());
+        }
+
+        if(this.access_str.equals("")){
+            this.UIAlert("Input error", "Please enter a TCP or UDP port", this);
             return;
         }
-
-        if (this.mAccessPort != null) {
-            int port;
-            try {
-                Integer.parseInt(this.mAccessPort.getText().toString());
-            } catch (Exception e) {
-                this.UIAlert("Input error", "Please enter a valid port number", this);
-                return;
-            }
-            this.access_str = this.access_str + "/" + mAccessPort.getText();
-            edit.putString("accessPort_str", mAccessPort.getText().toString());
-        }
-
+        
         if (this.mAllowip != null && this.mAllowip.getSelectedItem() != null && !this.mAllowip.getSelectedItem().toString().trim().equals("")) {
             if(mAllowip.getSelectedItem().toString().trim().equals("Source IP")) {
                 this.allowip_str = "0.0.0.0";
@@ -322,6 +346,14 @@ public class Fwknop extends Activity {
             return;
         }
 
+        if (this.mDestport != null && !this.mDestport.getText().toString().trim().equals("")) {
+            this.destport_str = mDestport.getText().toString();
+            edit.putString("destport_str", mDestport.getText().toString());
+        } else {
+            this.UIAlert("Input error", "Please enter a valid Server port", this);
+            return;
+        }
+
         if (this.mFwTimeout != null) {
             int fw_timeout;
             try {
@@ -349,26 +381,19 @@ public class Fwknop extends Activity {
     public void setupWidgets() {
 
         SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
-        this.mAccessPort = (EditText) findViewById(R.id.accessPort);
-        this.mAccessPort.setText(prefs.getString("accessPort_str", "22"));
-
-        this.mAccessProto = (Spinner) findViewById(R.id.accessProto);
-        String[] arraySpinner = new String[]{"tcp", "udp"};
-        ArrayAdapter adapter = new ArrayAdapter(this, android.R.layout.simple_spinner_item, arraySpinner);
-        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
-        this.mAccessProto.setAdapter(adapter);
-        if (prefs.getString("accessProto_str", "tcp").equals("tcp")) {
-            this.mAccessProto.setSelection(0);
-        }
-        if (prefs.getString("accessProto_str", "tcp").equals("udp")) {
-            this.mAccessProto.setSelection(1);
-        }
+        this.mTCPAccessPorts = (EditText) findViewById(R.id.tcpAccessPorts);
+        this.mTCPAccessPorts.setText(prefs.getString("tcpAccessPorts_str", "22"));
+        this.mUDPAccessPorts = (EditText) findViewById(R.id.udpAccessPorts);
+        this.mUDPAccessPorts.setText(prefs.getString("udpAccessPorts_str", null));
 
         this.mAllowip = (Spinner) findViewById(R.id.allowip);
 
         this.mDestip = (EditText) findViewById(R.id.destIP);
         this.mDestip.setText(prefs.getString("destip_str", ""));
 
+        this.mDestport = (EditText) findViewById(R.id.destPort);
+        this.mDestport.setText(prefs.getString("destport_str", "62201"));
+
         this.mFwTimeout = (EditText) findViewById(R.id.fwTimeout);
         this.mFwTimeout.setText(prefs.getString("fw_timeout_str", "60"));
 
@@ -376,9 +401,12 @@ public class Fwknop extends Activity {
         this.mCheck.setChecked(prefs.getBoolean("app_start", false));
 
         this.mPasswd = (EditText) findViewById(R.id.passwd);
+        this.mPasswd.setText(prefs.getString("passwd_str", ""));
+
         this.mOutput = (TextView) findViewById(R.id.output);
 
         this.mHmac   = (EditText) findViewById(R.id.hmac);
+        this.mHmac.setText(prefs.getString("hmac_str", ""));
 
         mUnlock = (ImageButton) findViewById(R.id.unlock);
         mUnlock.setOnClickListener(new OnClickListener() {
