@@ -1,13 +1,12 @@
-/*
- ******************************************************************************
+/**
+ * \file client/cmd_opts.h
  *
- * File:    cmd_opts.h
- *
- * Author:  Damien Stuart
- *
- * Purpose: Header file for fwknop command line options.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for fwknop command line options.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -39,6 +38,8 @@ enum {
     NAT_LOCAL,
     NAT_PORT,
     NAT_RAND_PORT,
+    NO_HOME_DIR,
+    NO_RC_FILE,
     TIME_OFFSET_MINUS,
     TIME_OFFSET_PLUS,
     SAVE_RC_STANZA,
@@ -47,8 +48,11 @@ enum {
     NO_SAVE_ARGS,
     SHOW_LAST_ARGS,
     RC_FILE_PATH,
+    RESOLVE_HTTP_ONLY,
     RESOLVE_URL,
+    SERVER_RESOLVE_IPV4,
     USE_HMAC,
+    USE_WGET_USER_AGENT,
     SPA_ICMP_TYPE,
     SPA_ICMP_CODE,
     KEY_LEN,
@@ -61,11 +65,14 @@ enum {
     KEY_HMAC,
     FD_SET_STDIN,
     FD_SET_ALT,
+    FAULT_INJECTION_TAG,
+
     /* Put GPG-related items below the following line */
     GPG_ENCRYPTION      = 0x200,
     GPG_RECIP_KEY,
     GPG_SIGNER_KEY,
     GPG_HOME_DIR,
+    GPG_EXE_PATH,
     GPG_AGENT,
     GPG_ALLOW_NO_SIGNING_PW,
     NOOP /* Just to be a marker for the end */
@@ -74,7 +81,7 @@ enum {
 
 /* Our getopt_long options string.
 */
-#define GETOPTS_OPTION_STRING "a:A:bB:C:D:E:f:gG:hH:kK:lm:M:n:N:p:P:Q:rRsS:Tu:U:vV"
+#define GETOPTS_OPTION_STRING "a:A:bB:C:D:E:f:gG:hH:kK:lm:M:n:N:p:P:Q:rRsS:Tu:U:vVw:"
 
 /* Our program command-line options...
 */
@@ -95,10 +102,12 @@ static struct option cmd_opts[] =
     {"encryption-mode",     1, NULL, ENCRYPTION_MODE},
     {"fd",                  1, NULL, FD_SET_ALT},
     {"fw-timeout",          1, NULL, 'f'},
+    {"fault-injection-tag", 1, NULL, FAULT_INJECTION_TAG },
     {"gpg-encryption",      0, NULL, 'g'},
     {"gpg-recipient-key",   1, NULL, GPG_RECIP_KEY },
     {"gpg-signer-key",      1, NULL, GPG_SIGNER_KEY },
     {"gpg-home-dir",        1, NULL, GPG_HOME_DIR },
+    {"gpg-exe",             1, NULL, GPG_EXE_PATH },
     {"gpg-agent",           0, NULL, GPG_AGENT },
     {"gpg-no-signing-pw",   0, NULL, GPG_ALLOW_NO_SIGNING_PW },
     {"get-key",             1, NULL, 'G'},
@@ -122,13 +131,19 @@ static struct option cmd_opts[] =
     {"nat-local",           0, NULL, NAT_LOCAL},
     {"nat-port",            1, NULL, NAT_PORT},
     {"nat-rand-port",       0, NULL, NAT_RAND_PORT},
+    {"no-home-dir",         0, NULL, NO_HOME_DIR},
+    {"no-rc-file",          0, NULL, NO_RC_FILE},
     {"server-port",         1, NULL, 'p'},
     {"server-proto",        1, NULL, 'P'},
-    {"spoof-src",           1, NULL, 'Q'},
+    {"spoof-source",        1, NULL, 'Q'},
+    {"spoof-src",           1, NULL, 'Q'}, /* synonym */
     {"rc-file",             1, NULL, RC_FILE_PATH},
     {"rand-port",           0, NULL, 'r'},
     {"resolve-ip-http",     0, NULL, 'R'},
+    {"resolve-ip-https",    0, NULL, 'R'}, /* synonym, default is HTTPS */
+    {"resolve-http-only",   0, NULL, RESOLVE_HTTP_ONLY},
     {"resolve-url",         1, NULL, RESOLVE_URL},
+    {"server-resolve-ipv4", 0, NULL, SERVER_RESOLVE_IPV4},
     {"show-last",           0, NULL, SHOW_LAST_ARGS},
     {"source-ip",           0, NULL, 's'},
     {"source-port",         1, NULL, 'S'},
@@ -138,9 +153,11 @@ static struct option cmd_opts[] =
     {"time-offset-minus",   1, NULL, TIME_OFFSET_MINUS},
     {"user-agent",          1, NULL, 'u'},
     {"use-hmac",            0, NULL, USE_HMAC},
+    {"use-wget-user-agent", 0, NULL, USE_WGET_USER_AGENT},
     {"spoof-user",          1, NULL, 'U'},
     {"verbose",             0, NULL, 'v'},
     {"version",             0, NULL, 'V'},
+    {"wget-cmd",            1, NULL, 'w'},
     {0, 0, 0, 0}
 };
 
