@@ -91,9 +91,13 @@
 
 /* Work out endianness
 */
-#if HAVE_ENDIAN_H       /* Should cover most Linux systems */
+#ifdef HAVE_ENDIAN_H
   #include <endian.h>
-  #define BYTEORDER __BYTE_ORDER
+  #if defined(BYTE_ORDER) /* POSIX proposal */
+    #define BYTEORDER BYTE_ORDER
+  #elif defined(__BYTE_ORDER) /* older systems? */
+    #define BYTEORDER __BYTE_ORDER
+  #endif
 #elif HAVE_SYS_ENDIAN_H /* FreeBSD has a sys/endian.h */
   #include <sys/endian.h>
   #define BYTEORDER _BYTE_ORDER
