@@ -1,15 +1,14 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_digest.c
  *
- * File:    fko_digest.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Create the base64-encoded digest for the current spa data. The
+ * \brief Create the base64-encoded digest for the current spa data. The
  *          digest used is determined by the digest_type setting in the
  *          fko context.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -40,11 +39,18 @@ static int
 set_spa_digest_type(fko_ctx_t ctx,
     short *digest_type_field, const short digest_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("set_spa_digest_type_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("set_spa_digest_type_val",
+            FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_VALIDFAIL);
+#endif
     if(digest_type < 1 || digest_type >= FKO_LAST_DIGEST_TYPE)
         return(FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_VALIDFAIL);
 
@@ -72,11 +78,20 @@ fko_set_raw_spa_digest_type(fko_ctx_t ctx, const short raw_digest_type)
 int
 fko_get_spa_digest_type(fko_ctx_t ctx, short *digest_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_digest_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_digest_type_val",
+            FKO_ERROR_INVALID_DATA);
+#endif
+
     if(digest_type == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
@@ -90,6 +105,10 @@ fko_get_spa_digest_type(fko_ctx_t ctx, short *digest_type)
 int
 fko_get_raw_spa_digest_type(fko_ctx_t ctx, short *raw_digest_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_raw_spa_digest_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -108,13 +127,23 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
 
     data_len = strnlen(data, MAX_SPA_ENCODED_MSG_SIZE);
 
+#if HAVE_LIBFIU
+    fiu_return_on("set_digest_toobig",
+            FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_TOOBIG);
+#endif
+
     if(data_len == MAX_SPA_ENCODED_MSG_SIZE)
         return(FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_TOOBIG);
 
+#if HAVE_LIBFIU
+    fiu_return_on("set_digest_invalidtype", FKO_ERROR_INVALID_DIGEST_TYPE);
+    fiu_return_on("set_digest_calloc", FKO_ERROR_MEMORY_ALLOCATION);
+#endif
+
     switch(digest_type)
     {
         case FKO_DIGEST_MD5:
-            md = malloc(MD_HEX_SIZE(MD5_DIGEST_LEN)+1);
+            md = calloc(1, MD_HEX_SIZE(MD5_DIGEST_LEN)+1);
             if(md == NULL)
                 return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -124,7 +153,7 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
             break;
 
         case FKO_DIGEST_SHA1:
-            md = malloc(MD_HEX_SIZE(SHA1_DIGEST_LEN)+1);
+            md = calloc(1, MD_HEX_SIZE(SHA1_DIGEST_LEN)+1);
             if(md == NULL)
                 return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -134,7 +163,7 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
             break;
 
         case FKO_DIGEST_SHA256:
-            md = malloc(MD_HEX_SIZE(SHA256_DIGEST_LEN)+1);
+            md = calloc(1, MD_HEX_SIZE(SHA256_DIGEST_LEN)+1);
             if(md == NULL)
                 return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -144,7 +173,7 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
             break;
 
         case FKO_DIGEST_SHA384:
-            md = malloc(MD_HEX_SIZE(SHA384_DIGEST_LEN)+1);
+            md = calloc(1, MD_HEX_SIZE(SHA384_DIGEST_LEN)+1);
             if(md == NULL)
                 return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -154,7 +183,7 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
             break;
 
         case FKO_DIGEST_SHA512:
-            md = malloc(MD_HEX_SIZE(SHA512_DIGEST_LEN)+1);
+            md = calloc(1, MD_HEX_SIZE(SHA512_DIGEST_LEN)+1);
             if(md == NULL)
                 return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -163,11 +192,31 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
             *digest_len = SHA512_B64_LEN;
             break;
 
+        case FKO_DIGEST_SHA3_256:
+            md = calloc(1, MD_HEX_SIZE(SHA3_256_DIGEST_LEN)+1);
+            if(md == NULL)
+                return(FKO_ERROR_MEMORY_ALLOCATION);
+
+            sha3_256_base64(md,
+                (unsigned char*)data, data_len);
+            *digest_len = SHA3_256_B64_LEN;
+            break;
+
+        case FKO_DIGEST_SHA3_512:
+            md = calloc(1, MD_HEX_SIZE(SHA3_512_DIGEST_LEN)+1);
+            if(md == NULL)
+                return(FKO_ERROR_MEMORY_ALLOCATION);
+
+            sha3_512_base64(md,
+                (unsigned char*)data, data_len);
+            *digest_len = SHA3_512_B64_LEN;
+            break;
+
         default:
             return(FKO_ERROR_INVALID_DIGEST_TYPE);
     }
 
-    /* Just in case this is a subsquent call to this function.  We
+    /* Just in case this is a subsequent call to this function.  We
      * do not want to be leaking memory.
     */
     if(*digest != NULL)
@@ -181,6 +230,9 @@ set_digest(char *data, char **digest, short digest_type, int *digest_len)
 int
 fko_set_spa_digest(fko_ctx_t ctx)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_digest_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -191,6 +243,10 @@ fko_set_spa_digest(fko_ctx_t ctx)
     if(ctx->encoded_msg == NULL)
         return(FKO_ERROR_MISSING_ENCODED_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_digest_encoded", FKO_ERROR_MISSING_ENCODED_DATA);
+#endif
+
     return set_digest(ctx->encoded_msg, &ctx->digest,
         ctx->digest_type, &ctx->digest_len);
 }
@@ -198,6 +254,9 @@ fko_set_spa_digest(fko_ctx_t ctx)
 int
 fko_set_raw_spa_digest(fko_ctx_t ctx)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_raw_spa_digest_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -208,6 +267,10 @@ fko_set_raw_spa_digest(fko_ctx_t ctx)
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MISSING_ENCODED_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_raw_spa_digest_val", FKO_ERROR_MISSING_ENCODED_DATA);
+#endif
+
     return set_digest(ctx->encrypted_msg, &ctx->raw_digest,
         ctx->raw_digest_type, &ctx->raw_digest_len);
 }
@@ -215,11 +278,17 @@ fko_set_raw_spa_digest(fko_ctx_t ctx)
 int
 fko_get_spa_digest(fko_ctx_t ctx, char **md)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_digest_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_digest_val", FKO_ERROR_INVALID_DATA);
+#endif
     if(md == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
@@ -231,6 +300,9 @@ fko_get_spa_digest(fko_ctx_t ctx, char **md)
 int
 fko_get_raw_spa_digest(fko_ctx_t ctx, char **md)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_raw_spa_digest_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
