@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_funcs.c
  *
- * File:    fko_funcs.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: General utility functions for libfko
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief General utility functions for libfko
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -43,37 +42,37 @@ fko_new(fko_ctx_t *r_ctx)
     int         res;
     char       *ver;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_new_calloc", FKO_ERROR_MEMORY_ALLOCATION);
+#endif
+
     ctx = calloc(1, sizeof *ctx);
     if(ctx == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
     /* Set default values and state.
      *
-     * Note: We have to explicitly set the ctx->state to initialized
-     *       just before making an fko_xxx function call, then set it
-     *       back to zero just afer.  During initialization, we need
-     *       to make these functions think they are operating on an
-     *       initialized context, or else they would fail.
+     * Note: We initialize the context early so that the fko_set_xxx
+     *       functions can operate properly. If there are any problems during
+     *       initialization, then fko_destroy() is called which will clean up
+     *       the context.
     */
+    ctx->initval = FKO_CTX_INITIALIZED;
 
     /* Set the version string.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     ver = strdup(FKO_PROTOCOL_VERSION);
-    ctx->initval = 0;
     if(ver == NULL)
     {
-        free(ctx);
+        fko_destroy(ctx);
+        ctx = NULL;
         return(FKO_ERROR_MEMORY_ALLOCATION);
     }
-
     ctx->version = ver;
 
     /* Rand value.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_rand_value(ctx, NULL);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -83,9 +82,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Username.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_username(ctx, NULL);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -95,9 +92,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Timestamp.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_timestamp(ctx, 0);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -107,9 +102,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Default Digest Type.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_spa_digest_type(ctx, FKO_DEFAULT_DIGEST);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -119,9 +112,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Default Message Type.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_spa_message_type(ctx, FKO_DEFAULT_MSG_TYPE);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -131,9 +122,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Default Encryption Type.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_spa_encryption_type(ctx, FKO_DEFAULT_ENCRYPTION);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -143,9 +132,7 @@ fko_new(fko_ctx_t *r_ctx)
 
     /* Default is Rijndael in CBC mode
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_spa_encryption_mode(ctx, FKO_DEFAULT_ENC_MODE);
-    ctx->initval = 0;
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
@@ -160,10 +147,6 @@ fko_new(fko_ctx_t *r_ctx)
 
 #endif /* HAVE_LIBGPGME */
 
-    /* Now we mean it.
-    */
-    ctx->initval = FKO_CTX_INITIALIZED;
-
     FKO_SET_CTX_INITIALIZED(ctx);
 
     *r_ctx = ctx;
@@ -185,9 +168,19 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
     int         res = FKO_SUCCESS; /* Are we optimistic or what? */
     int         enc_msg_len;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_new_with_data_msg",
+            FKO_ERROR_INVALID_DATA_FUNCS_NEW_ENCMSG_MISSING);
+#endif
+
     if(enc_msg == NULL)
         return(FKO_ERROR_INVALID_DATA_FUNCS_NEW_ENCMSG_MISSING);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_new_with_data_keylen",
+            FKO_ERROR_INVALID_KEY_LEN);
+#endif
+
     if(dec_key_len < 0 || hmac_key_len < 0)
         return(FKO_ERROR_INVALID_KEY_LEN);
 
@@ -203,9 +196,6 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
         return(FKO_ERROR_INVALID_DATA_FUNCS_NEW_MSGLEN_VALIDFAIL);
     }
 
-    if(ctx->encrypted_msg != NULL)
-        free(ctx->encrypted_msg);
-
     /* First, add the data to the context.
     */
     ctx->encrypted_msg     = strdup(enc_msg);
@@ -227,11 +217,9 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
         ctx = NULL;
         return res;
     }
-    ctx->initval = 0;
 
     /* HMAC digest type
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     res = fko_set_spa_hmac_type(ctx, hmac_type);
     if(res != FKO_SUCCESS)
     {
@@ -239,11 +227,9 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
         ctx = NULL;
         return res;
     }
-    ctx->initval = 0;
 
     /* Check HMAC if the access stanza had an HMAC key
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     if(hmac_key_len > 0 && hmac_key != NULL)
         res = fko_verify_hmac(ctx, hmac_key, hmac_key_len);
     if(res != FKO_SUCCESS)
@@ -252,11 +238,9 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
         ctx = NULL;
         return res;
     }
-    ctx->initval = 0;
 
     /* Consider it initialized here.
     */
-    ctx->initval = FKO_CTX_INITIALIZED;
     FKO_SET_CTX_INITIALIZED(ctx);
 
     /* If a decryption key is provided, go ahead and decrypt and decode.
@@ -452,6 +436,11 @@ fko_base64_decode(const char * const in, unsigned char *out)
 int
 fko_get_version(fko_ctx_t ctx, char **version)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_version_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -460,6 +449,10 @@ fko_get_version(fko_ctx_t ctx, char **version)
     if(version == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_version_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *version = ctx->version;
 
     return(FKO_SUCCESS);
@@ -527,6 +520,11 @@ fko_spa_data_final(fko_ctx_t ctx,
 int
 fko_get_spa_data(fko_ctx_t ctx, char **spa_data)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_data_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -535,12 +533,20 @@ fko_get_spa_data(fko_ctx_t ctx, char **spa_data)
     if(spa_data == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_data_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     /* We expect to have encrypted data to process.  If not, we bail.
     */
     if(ctx->encrypted_msg == NULL || ! is_valid_encoded_msg_len(
                 strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE)))
         return(FKO_ERROR_MISSING_ENCODED_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_data_encoded", FKO_ERROR_MISSING_ENCODED_DATA);
+#endif
+
     *spa_data = ctx->encrypted_msg;
 
     /* Notice we omit the first 10 bytes if Rijndael encryption is
@@ -590,4 +596,39 @@ fko_set_spa_data(fko_ctx_t ctx, const char * const enc_msg)
     return(FKO_SUCCESS);
 }
 
+#if AFL_FUZZING
+/* provide a way to set the encrypted data directly without base64 encoding.
+ * This allows direct AFL fuzzing against decryption routines.
+*/
+int
+fko_afl_set_spa_data(fko_ctx_t ctx, const char * const enc_msg, const int enc_msg_len)
+{
+    /* Must be initialized
+    */
+    if(!CTX_INITIALIZED(ctx))
+        return FKO_ERROR_CTX_NOT_INITIALIZED;
+
+    if(enc_msg == NULL)
+        return(FKO_ERROR_INVALID_DATA_FUNCS_SET_MSGLEN_VALIDFAIL);
+
+    if(! is_valid_encoded_msg_len(enc_msg_len))
+        return(FKO_ERROR_INVALID_DATA_FUNCS_SET_MSGLEN_VALIDFAIL);
+
+    if(ctx->encrypted_msg != NULL)
+        free(ctx->encrypted_msg);
+
+    /* Copy the raw encrypted data into the context
+    */
+    ctx->encrypted_msg = calloc(1, enc_msg_len);
+    if(ctx->encrypted_msg == NULL)
+        return(FKO_ERROR_MEMORY_ALLOCATION);
+
+    memcpy(ctx->encrypted_msg, enc_msg, enc_msg_len);
+
+    ctx->encrypted_msg_len = enc_msg_len;
+
+    return(FKO_SUCCESS);
+}
+#endif
+
 /***EOF***/
