@@ -1,11 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_limits.h
  *
- * File:    fko_limits.h
- *
- * Author:  Damien S. Stuart
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief #defines for libfko limits
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -53,13 +54,14 @@
 
 #define MIN_GNUPG_MSG_SIZE          400
 #define MIN_SPA_FIELDS                6
-#define MAX_SPA_FIELDS               10
+#define MAX_SPA_FIELDS                9
 
 #define MAX_IPV4_STR_LEN             16
 #define MIN_IPV4_STR_LEN              7
 
-#define MAX_PROTO_STR_LEN   		  4  /* tcp, udp, icmp for now */
-#define MAX_PORT_STR_LEN    		  5
+#define MAX_PROTO_STR_LEN             4  /* tcp, udp, icmp for now */
+#define MAX_PORT_STR_LEN              5
+#define MAX_PORT                  65535
 
 /* Misc.
 */
