@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_nat_access.c
  *
- * File:    fko_nat_access.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Set/Get the spa nat access request data.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Set/Get the spa nat access request data.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -38,6 +37,10 @@ fko_set_spa_nat_access(fko_ctx_t ctx, const char * const msg)
 {
     int res = FKO_SUCCESS;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_nat_access_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Context must be initialized.
     */
     if(!CTX_INITIALIZED(ctx))
@@ -48,16 +51,24 @@ fko_set_spa_nat_access(fko_ctx_t ctx, const char * const msg)
     if(msg == NULL || strnlen(msg, MAX_SPA_NAT_ACCESS_SIZE) == 0)
         return(FKO_ERROR_INVALID_DATA_NAT_EMPTY);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_nat_access_empty", FKO_ERROR_INVALID_DATA_NAT_EMPTY);
+#endif
+
     /* --DSS XXX: Bail out for now.  But consider just
      *            truncating in the future...
     */
     if(strnlen(msg, MAX_SPA_NAT_ACCESS_SIZE) == MAX_SPA_NAT_ACCESS_SIZE)
         return(FKO_ERROR_DATA_TOO_LARGE);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_nat_access_large", FKO_ERROR_DATA_TOO_LARGE);
+#endif
+
     if((res = validate_nat_access_msg(msg)) != FKO_SUCCESS)
         return(res);
 
-    /* Just in case this is a subsquent call to this function.  We
+    /* Just in case this is a subsequent call to this function.  We
      * do not want to be leaking memory.
     */
     if(ctx->nat_access != NULL)
@@ -81,7 +92,8 @@ fko_set_spa_nat_access(fko_ctx_t ctx, const char * const msg)
             ctx->message_type = FKO_CLIENT_TIMEOUT_NAT_ACCESS_MSG;
     }
     else
-        if(ctx->message_type != FKO_LOCAL_NAT_ACCESS_MSG)
+        if(ctx->message_type != FKO_LOCAL_NAT_ACCESS_MSG
+                && ctx->message_type != FKO_CLIENT_TIMEOUT_LOCAL_NAT_ACCESS_MSG)
             ctx->message_type = FKO_NAT_ACCESS_MSG;
 
     return(FKO_SUCCESS);
@@ -92,6 +104,11 @@ fko_set_spa_nat_access(fko_ctx_t ctx, const char * const msg)
 int
 fko_get_spa_nat_access(fko_ctx_t ctx, char **nat_access)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_nat_access_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -100,6 +117,10 @@ fko_get_spa_nat_access(fko_ctx_t ctx, char **nat_access)
     if(nat_access == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_nat_access_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *nat_access = ctx->nat_access;
 
     return(FKO_SUCCESS);
