@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/gpgme_funcs.c
  *
- * File:    gpgme_funcs.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: gpgme-related functions for GPG encryptions support in libfko.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief gpgme-related functions for GPG encryptions support in libfko.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -107,7 +106,7 @@ my_passphrase_cb(
 
 /* Verify gpg signatures in a verify_result set.
 */
-int
+static int
 process_sigs(fko_ctx_t fko_ctx, gpgme_verify_result_t vres)
 {
     unsigned int        sig_cnt = 0;
@@ -397,7 +396,7 @@ gpgme_encrypt(fko_ctx_t fko_ctx, unsigned char *indata, size_t in_len,
     */
     tmp_buf = gpgme_data_release_and_get_mem(cipher, out_len);
 
-    *out = malloc(*out_len); /* This is freed upon fko_ctx destruction. */
+    *out = calloc(1, *out_len); /* This is freed upon fko_ctx destruction. */
     if(*out == NULL)
         res = FKO_ERROR_MEMORY_ALLOCATION;
     else
