@@ -1,11 +1,10 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/md5.c
  *
- * File:    md5.c
- *
- * Purpose: Implementation of the MD5 message-digest algorithm for libfwknop.
- *
- * This code implements the MD5 message-digest algorithm.
+ * \brief Implementation of the MD5 message-digest algorithm for libfwknop.
+ */
+
+/* This code implements the MD5 message-digest algorithm.
  *
  * The algorithm is due to Ron Rivest. This code was
  * written by Colin Plumb in 1993, no copyright is claimed.
@@ -28,6 +27,7 @@
  *****************************************************************************
 */
 #include "md5.h"
+#include "fko_common.h"
 
 #if BYTEORDER == 1234
   #define byteReverse(buf, len)    /* Nothing */
@@ -118,7 +118,7 @@ MD5Update(MD5Context *ctx, unsigned char *buf, unsigned len)
     memcpy(ctx->in, buf, len);
 }
 
-/* Final wrapup - pad to 64-byte boundary with the bit pattern 
+/* Final wrapup - pad to 64-byte boundary with the bit pattern
  * 1 0* (64-bit count of bits processed, MSB-first)
  */
 void
@@ -163,13 +163,14 @@ MD5Final(unsigned char digest[16], MD5Context *ctx)
 
     /* Append length in bits and transform
     */
-    ((uint32_t *) ctx->in)[14] = ctx->bits[0];
-    ((uint32_t *) ctx->in)[15] = ctx->bits[1];
+    memcpy(&(ctx->in[56]), &(ctx->bits[0]), sizeof(uint32_t));
+    memcpy(&(ctx->in[60]), &(ctx->bits[1]), sizeof(uint32_t));
 
     MD5Transform(ctx->buf, (uint32_t *) ctx->in);
     byteReverse((unsigned char *) ctx->buf, 4);
     memcpy(digest, ctx->buf, 16);
-    memset(ctx, 0, sizeof(ctx));        /* In case it's sensitive */
+
+    memset(ctx, 0, sizeof(*ctx));        /* In case it's sensitive */
 }
 
 
@@ -271,6 +272,6 @@ MD5Transform(uint32_t buf[4], uint32_t in[16])
     buf[1] += b;
     buf[2] += c;
     buf[3] += d;
-} 
+}
 
 /***EOF***/
