@@ -3,10 +3,29 @@
  *
  * File:    fkomodule.c
  *
- * Author:  Damien S. Stuart <dstuart@dstuart.org>
- *
  * Purpose: Python wrapper module for the fwknop library (libfko).
  *
+ *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
+ *
+ *  License (GNU General Public License):
+ *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License
+ *  as published by the Free Software Foundation; either version 2
+ *  of the License, or (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  USA
+ *
  *****************************************************************************
 */
 #include <Python.h>
@@ -1280,14 +1299,13 @@ base64_encode(PyObject *self, PyObject *args)
     unsigned char *in;
     int in_len;
     char *out;
-    int res;
 
     /* --DSS Note the order of args is different than the libfko call.
              We need to do this for the following parse call. */
     if(!PyArg_ParseTuple(args, "s#s", &in, &in_len, &out))
         return NULL;
 
-    res = fko_base64_encode(in, out, in_len);
+    fko_base64_encode(in, out, in_len);
 
     return Py_BuildValue("s", out);
 }
@@ -1355,13 +1373,13 @@ static PyObject *
 get_spa_hmac(PyObject *self, PyObject *args)
 {
     fko_ctx_t ctx;
-    char *enc_data;
+    char *hmac_data;
     int res;
 
     if(!PyArg_ParseTuple(args, "k", &ctx))
         return NULL;
 
-    res = fko_get_spa_hmac(ctx, &enc_data);
+    res = fko_get_spa_hmac(ctx, &hmac_data);
 
     if(res != FKO_SUCCESS)
     {
@@ -1369,7 +1387,7 @@ get_spa_hmac(PyObject *self, PyObject *args)
         return NULL;
     }
 
-    return Py_BuildValue("s", enc_data);
+    return Py_BuildValue("s", hmac_data);
 }
 
 /*****************************************************************************
