@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/fw_util_ipf.h
  *
- * File:    fw_util_ipf.h
- *
- * Author:  Damien Stuart (dstuart@dstuart.org)
- *
- * Purpose: Header file for fw_util_ipf.c.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for fw_util_ipf.c.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -40,6 +39,7 @@
 #define IPF_ADD_SNAT_RULE_ARGS ""
 #define IPF_DEL_RULE_ARGS ""
 #define IPF_LIST_RULES_ARGS ""
+#define IPF_ANY_IP ""
 
 #endif /* FW_UTIL_IPF_H */
 
