@@ -1,14 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/fw_util_pf.h
  *
- * File:    fw_util_pf.h
- *
- * Author:  Damien Stuart (dstuart@dstuart.org), Michael Rash
- *                                                  (mbr@cipherdyne.org)
- *
- * Purpose: Header file for fw_util_pf.c.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for fw_util_pf.c.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -33,15 +31,26 @@
 #define FW_UTIL_PF_H
 
 #define MAX_PF_ANCHOR_SEARCH_LEN    (MAX_PF_ANCHOR_LEN+11)   /* room for 'anchor "' string */
-#define MAX_PF_NEW_RULE_LEN 120
+#define MAX_PF_NEW_RULE_LEN         140
+
+#if HAVE_EXECVPE
+  #define SH_REDIR "" /* the shell is not used when execvpe() is available */
+#else
+  #define SH_REDIR " 2>&1"
+#endif
 
 /* pf command args
 */
-#define PF_ADD_RULE_ARGS              "pass in quick proto %u from %s to any port %u keep state label " EXPIRE_COMMENT_PREFIX "%u"
+#define PF_ADD_RULE_ARGS              "pass in quick proto %u from %s to %s port %u keep state label " EXPIRE_COMMENT_PREFIX "%u"
 #define PF_WRITE_ANCHOR_RULES_ARGS    "-a %s -f -"
-#define PF_LIST_ANCHOR_RULES_ARGS     "-a %s -s rules 2>&1"
-#define PF_ANCHOR_CHECK_ARGS          "-s Anchor 2>&1"  /* to check for fwknop anchor */
-#define PF_DEL_ALL_ANCHOR_RULES       "-a %s -F all 2>&1"
+#if HAVE_EXECVPE
+  #define PF_LIST_ANCHOR_RULES_ARGS   "-a %s -s rules"
+#else
+  #define PF_LIST_ANCHOR_RULES_ARGS   "-a %s -s rules 2> /dev/null"
+#endif
+#define PF_ANCHOR_CHECK_ARGS          "-s Anchor" SH_REDIR  /* to check for fwknop anchor */
+#define PF_DEL_ALL_ANCHOR_RULES       "-a %s -F all" SH_REDIR
+#define PF_ANY_IP                     "any"
 
 #endif /* FW_UTIL_PF_H */
 
