@@ -112,7 +112,7 @@ preprocess_spa_data(const fko_srv_options_t *opts, spa_pkt_info_t *spa_pkt, spa_
             xff += 17;
 
             for (i = 0; *xff != '\0'; i++)
-                if (isspace(*xff))
+                if (isspace((int)(unsigned char)*xff))
                    *xff = '\0';
                 else
                    xff++;
@@ -135,7 +135,7 @@ preprocess_spa_data(const fko_srv_options_t *opts, spa_pkt_info_t *spa_pkt, spa_
 
         for(i=0; i<pkt_data_len; i++)
         {
-            if(isspace(*ndx)) /* The first space marks the end of the req */
+            if(isspace((int)(unsigned char)*ndx)) /* The first space marks the end of the req */
             {
                 *ndx = '\0';
                 break;
@@ -874,7 +874,16 @@ set_timeout(acc_stanza_t *acc, spa_data_t *spadat)
     spadat->fw_access_timeout = DEF_FW_ACCESS_TIMEOUT;
 
     if(spadat->client_timeout > 0)
-        spadat->fw_access_timeout = spadat->client_timeout;
+        if(acc->max_fw_timeout < spadat->client_timeout)
+        {
+            /* don't allow clients to request more time than the max
+            */
+            spadat->fw_access_timeout = acc->max_fw_timeout;
+        }
+        else
+        {
+            spadat->fw_access_timeout = spadat->client_timeout;
+        }
     else if(acc->fw_access_timeout > 0)
         spadat->fw_access_timeout = acc->fw_access_timeout;
 
