@@ -1,16 +1,17 @@
-/*
- *****************************************************************************
+/**
+ * \file server/process_packet.c
  *
- * File:    process_packet.c
+ * \brief Packet parser/decoder for fwknopd server.
  *
- * Author:  Damien S. Stuart
- *
- * Purpose: Packet parser/decoder for fwknopd server.  Takes the raw packet
+ *          Takes the raw packet
  *          data from libpcap and parses/extracts the packet data payload,
  *          then creates an FKO context with that data.  If the context
  *          creation is successful, it is queued for processing.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -31,17 +32,18 @@
  *
  *****************************************************************************
 */
-#include <pcap.h>
 
 #include "fwknopd_common.h"
 #include "netinet_common.h"
 #include "process_packet.h"
 #include "incoming_spa.h"
 #include "utils.h"
+#include "log_msg.h"
+
 
 void
-process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
-    const unsigned char *packet)
+process_packet(PROCESS_PKT_ARGS_TYPE *args, PACKET_HEADER_META,
+               const unsigned char *packet)
 {
     struct ether_header *eth_p;
     struct iphdr        *iph_p;
@@ -52,6 +54,7 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
     unsigned char       *pkt_data;
     unsigned short      pkt_data_len;
     unsigned char       *pkt_end;
+    unsigned char       *fr_end;
 
     unsigned int        ip_hdr_words;
 
@@ -68,8 +71,25 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
 
     int                 offset = opts->data_link_offset;
 
+#if USE_LIBPCAP
     unsigned short      pkt_len = packet_header->len;
 
+    /* Gotta have a complete ethernet header.
+    */
+    if (packet_header->caplen < ETHER_HDR_LEN)
+        return;
+
+    /* Determine packet end.
+    */
+    fr_end = (unsigned char *) packet + packet_header->caplen;
+#else
+    /* This is coming from NFQ and we get the packet lentgh as an arg.
+    */
+    if (pkt_len < ETHER_HDR_LEN)
+        return;
+    fr_end = (unsigned char *) packet + pkt_len;
+#endif
+
     /* This is a hack to determine if we are using the linux cooked
      * interface.  We base it on the offset being 16 which is the
      * value it would be if the datalink is DLT_LINUX_SLL.  I don't
@@ -77,19 +97,10 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
     */
     unsigned char       assume_cooked = (offset == 16 ? 1 : 0);
 
-    /* Determine packet end.
-    */
-    pkt_end = (unsigned char *) packet + packet_header->caplen;
-
     /* The ethernet header.
     */
     eth_p = (struct ether_header*) packet;
 
-    /* Gotta have a complete ethernet header.
-    */
-    if (packet_header->caplen < ETHER_HDR_LEN)
-        return;
-
     eth_type = ntohs(*((unsigned short*)&eth_p->ether_type));
 
     if(eth_type == 0x8100) /* 802.1q encapsulated */
@@ -122,7 +133,7 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
 
     /* If IP header is past calculated packet end, bail.
     */
-    if ((unsigned char*)(iph_p + 1) > pkt_end)
+    if ((unsigned char*)(iph_p + 1) > fr_end)
         return;
 
     /* ip_hdr_words is the number of 32 bit words in the IP header. After
@@ -134,6 +145,15 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
     if (ip_hdr_words < MIN_IPV4_WORDS)
         return;
 
+    /* Make sure to calculate the packet end based on the length in the
+     * IP header. This allows additional bytes that may be added to the
+     * frame (such as a 4-byte Ethernet Frame Check Sequence) to not
+     * interfere with SPA operations.
+    */
+    pkt_end = ((unsigned char*)iph_p)+ntohs(iph_p->tot_len);
+    if(pkt_end > fr_end)
+        return;
+
     /* Now, find the packet data payload (depending on IPPROTO).
     */
     src_ip = iph_p->saddr;
@@ -213,4 +233,5 @@ process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header,
     return;
 }
 
+
 /***EOF***/
