@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/process_packet.h
  *
- * File:    process_packet.h
- *
- * Author:  Damien Stuart (dstuart@dstuart.org)
- *
- * Purpose: Header file for process_packet and other fwknopd code.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for process_packet and other fwknopd code.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -31,6 +30,15 @@
 #ifndef PROCESS_PACKET_H
 #define PROCESS_PACKET_H
 
+#if USE_LIBPCAP
+  #include <pcap.h>
+  #define PACKET_HEADER_META const struct pcap_pkthdr *packet_header
+  #define PROCESS_PKT_ARGS_TYPE unsigned char
+#else
+  #define PACKET_HEADER_META unsigned short pkt_len
+  #define PROCESS_PKT_ARGS_TYPE fko_srv_options_t
+#endif
+
 #define IPV4_VER_MASK   0x15
 #define MIN_IPV4_WORDS  0x05
 
@@ -45,6 +53,7 @@
 
 /* Prototypes
 */
-void process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header, const unsigned char *packet);
+void process_packet(PROCESS_PKT_ARGS_TYPE *opts, PACKET_HEADER_META,
+					const unsigned char *packet);
 
 #endif  /* PROCESS_PACKET_H */
