@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/sig_handler.h
  *
- * File:    sig_handler.h
- *
- * Author:  Damien Stuart (dstuart@dstuart.org)
- *
- * Purpose: Header file for sig_handler functions and data.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for sig_handler functions and data.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -44,6 +43,7 @@ extern sig_atomic_t got_sigchld;
 
 void sig_handler(int sig);
 int set_sig_handlers(void);
+int sig_do_stop(void);
 
 #endif /* SIG_HANDLER_H */
 
