@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/utils.h
  *
- * File:    utils.h
- *
- * Author:  Damien Stuart (dstuart@dstuart.org)
- *
- * Purpose: Header file for utils.c fwknopd server program.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for utils.c fwknopd server program.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -54,11 +53,18 @@
     x == '#' || x == '\n' || x == '\r' || x == ';' || x == '\0' \
 )
 
+#define IS_DIR  1
+#define IS_EXE  2
+#define IS_FILE 3
+
 /* Prototypes
 */
-void hex_dump(const unsigned char *data, const int size);
 char* dump_ctx(fko_ctx_t ctx);
-int is_valid_dir(const char *path);
-int verify_file_perms_ownership(const char *file);
+int   is_valid_dir(const char *path);
+int   is_valid_exe(const char *path);
+int   is_valid_file(const char *path);
+int   verify_file_perms_ownership(const char *file);
+void  truncate_partial_line(char *str);
+int   is_digits(const char * const str);
 
 #endif  /* UTILS_H */
