@@ -9,6 +9,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <string.h>
 #include "fko.h"
 
 #define ENABLE_GPG_TESTS 0
@@ -23,11 +24,33 @@
 #define NO_DIGEST        0
 #define DO_DIGEST        1
 #define RAW_DIGEST       2
-#define ENC_KEY          "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" /* 32 bytes */
-#define HMAC_KEY         "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB" /* 32 bytes */
+#define MAX_LINE_LEN     3000 /* really long for fuzzing tests */
+
+/* We use HMAC and encryption keys that are as long as the max
+ * sizes since we pass in bogus key lengths. That is, libfko
+ * functions cannot be expected to handle key lengths that are
+ * longer than the key buffers themselves.
+*/
+#define ENC_KEY         "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" /* 32 bytes (RIJNDAEL_MAX_KEYSIZE) */
+#define HMAC_KEY        "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB" /* 136 bytes (SHA3_256_BLOCK_LEN) */
+#define STR_8BYTES      "12345678"
+#define STR_16BYTES     "1234567890123456"
+#define STR_64BYTES     "1234567890123456789012345678901234567890123456789012345678901234"
+#define SPA_MSG1        "1.1.1.1,tcp/22"
+#define SPA_MSG2        "123.123.123.123,tcp/22"
+#define SPA_NAT_MSG     "1.2.3.4,1234"
+#define SERVER_AUTH_MSG "passwd"
+
+#define IS_EMPTY_LINE(x) ( \
+    x == '#' || x == '\n' || x == '\r' || x == ';' || x == '\0' \
+)
 
 static void display_ctx(fko_ctx_t ctx);
 static void test_loop(int new_ctx_flag, int destroy_ctx_flag);
+static void test_loop_compounded(void);
+#if FUZZING_INTERFACES
+static void spa_encoded_msg_fuzzing(void);
+#endif
 static void ctx_update(fko_ctx_t *ctx, int new_ctx_flag,
         int destroy_ctx_flag, int print_flag);
 static void spa_default_ctx(fko_ctx_t *ctx);
@@ -47,31 +70,313 @@ static void spa_func_getset_short(fko_ctx_t *ctx, char *set_name,
         int new_ctx_flag, int destroy_ctx_flag);
 
 int spa_calls = 0;
+int spa_compounded_calls = 0;
 
 int main(void) {
-    int i;
 
     test_loop(NO_NEW_CTX, NO_CTX_DESTROY);
     test_loop(NEW_CTX, CTX_DESTROY);
     test_loop(NEW_CTX, NO_CTX_DESTROY);
     test_loop(NO_NEW_CTX, CTX_DESTROY);
 
-    /* call fko_errstr() across valid and invalid values
-    */
-    for (i=-5; i < FKO_LAST_ERROR+5; i++)
-        printf("libfko error (%d): %s\n", i, fko_errstr(i));
+    printf("\n[+] Total libfko function calls (before compounded tests): %d\n\n",
+            spa_calls);
 
-    printf("\n[+] Total libfko function calls: %d\n\n", spa_calls);
+    printf("[+] Running compounded tests via: test_loop_compounded()...\n");
+    test_loop_compounded();
+
+    printf("\n[+] Total compounded function calls: %d\n", spa_compounded_calls);
+    printf("[+] Total libfko function calls (after compounded tests): %d\n\n",
+            spa_calls);
+
+#if FUZZING_INTERFACES
+    printf("[+] libfko fuzzing by setting SPA buffer manually...\n");
+    spa_encoded_msg_fuzzing();
+#endif
 
     return 0;
 }
 
+#if FUZZING_INTERFACES
+static void
+spa_encoded_msg_fuzzing(void)
+{
+    fko_ctx_t      decode_ctx = NULL, decrypt_ctx = NULL;
+    int            res = 0, pkt_id, require_success, require_digest, digest_type, msg_len;
+    int            line_ctr = 0, spa_payload_ctr = 0;
+    FILE          *fz  = NULL;
+    char           line[MAX_LINE_LEN] = {0};
+    char          *spa_data_final = NULL, *spa_data_copy = NULL;
+    char           b64_encoded_msg[MAX_LINE_LEN] = {0};
+    unsigned char  b64_decoded_msg[MAX_LINE_LEN] = {0};
+
+    /* fuzzing file contents are formatted like this (and are generated by the spa_fuzzing.py
+     * python fuzzer):
+     *
+     * <pkt_ID> <status: success|fail> <digest: yes|no> <digest type> <base64_SPA_payload>
+    */
+
+    if ((fz = fopen("fuzz_spa_payloads", "r")) == NULL)
+        return;
+
+    while ((fgets(line, MAX_LINE_LEN, fz)) != NULL)
+    {
+        line_ctr++;
+        line[MAX_LINE_LEN-1] = '\0';
+
+        if (line[strlen(line)-1] == '\n')
+            line[strlen(line)-1] = '\0';
+
+        if(IS_EMPTY_LINE(line[0]))
+            continue;
+
+        if(sscanf(line, "%d %d %d %d %s", &pkt_id, &require_success,
+                    &require_digest, &digest_type, b64_encoded_msg) != 5)
+        {
+            printf("[+] fuzzing parsing error at line: %d\n", line_ctr);
+            continue;
+        }
+
+        msg_len = fko_base64_decode(b64_encoded_msg, b64_decoded_msg);
+
+        spa_payload_ctr++;
+
+        fko_new(&decode_ctx);
+
+        if ((res = fko_set_encoded_data(decode_ctx, (char *) b64_decoded_msg,
+                        msg_len, require_digest, digest_type)) != FKO_SUCCESS) {
+            printf("[-] pkt_id: %d, fko_set_encoded_data(): %s\n", pkt_id, fko_errstr(res));
+        }
+
+        res = fko_decode_spa_data(decode_ctx);
+        if (require_success) {
+            if (res != FKO_SUCCESS) {
+                printf("[-] pkt_id: %d, expected decode success but: fko_decode_spa_data(): %s\n",
+                        pkt_id, fko_errstr(res));
+            }
+        } else {
+            if (res == FKO_SUCCESS) {
+                printf("[-] pkt_id: %d, expected decode failure but: fko_decode_spa_data(): %s\n",
+                        pkt_id, fko_errstr(res));
+            }
+        }
+
+        if (0) {
+                fko_destroy(decode_ctx);
+                decode_ctx = NULL;
+        }
+
+        fko_set_spa_hmac_type(decode_ctx, FKO_HMAC_SHA256);
+
+        res = fko_spa_data_final(decode_ctx, ENC_KEY, strlen(ENC_KEY),
+                    HMAC_KEY, strlen(HMAC_KEY));
+
+        if (res == FKO_SUCCESS) {
+            if ((res = fko_get_spa_data(decode_ctx, &spa_data_final)) != FKO_SUCCESS) {
+                printf("fko_get_spa_data(): %s\n", fko_errstr(res));
+            } else {
+                printf("PKT_ID: %d, PKT: %s\n", pkt_id, spa_data_final);
+
+                spa_data_copy = strdup(spa_data_final);
+
+                if (spa_data_final != NULL)
+                    memset(spa_data_final, 0x0, strlen(spa_data_final));
+                fko_destroy(decode_ctx);
+                decode_ctx = NULL;
+
+                res = fko_new_with_data(&decrypt_ctx, spa_data_copy, ENC_KEY,
+                    strlen(ENC_KEY), FKO_ENC_MODE_CBC, HMAC_KEY, strlen(HMAC_KEY),
+                    FKO_HMAC_SHA256);
+
+                if (spa_data_copy != NULL)
+                {
+                    memset(spa_data_copy, 0x0, strlen(spa_data_copy));
+                    free(spa_data_copy);
+                }
+
+                if (res == FKO_SUCCESS) {
+                    printf("pkt_id: %d Success\n", pkt_id);
+                } else {
+                    printf("fko_new_with_data(): %s\n", fko_errstr(res));
+                }
+
+                fko_destroy(decrypt_ctx);
+                decrypt_ctx = NULL;
+            }
+        } else {
+            printf("fko_spa_data_final(): %s\n", fko_errstr(res));
+            fko_destroy(decode_ctx);
+            decode_ctx = NULL;
+        }
+
+        memset(line, 0x0, MAX_LINE_LEN);
+        memset(b64_encoded_msg, 0x0, MAX_LINE_LEN);
+        memset(b64_decoded_msg, 0x0, MAX_LINE_LEN);
+    }
+
+    fclose(fz);
+
+    printf("[+] Sent %d SPA payloads through libfko encode/decode cycle...\n",
+            spa_payload_ctr);
+    return;
+}
+#endif
+
+static void ctx_set_null_vals(fko_ctx_t *ctx)
+{
+    fko_set_rand_value(*ctx, NULL);
+    fko_set_username(*ctx, NULL);
+    fko_set_spa_message(*ctx, NULL);
+    fko_set_spa_nat_access(*ctx, NULL);
+    fko_set_spa_server_auth(*ctx, NULL);
+    fko_set_spa_data(*ctx, NULL);
+    fko_set_timestamp(*ctx, 0);
+    fko_set_spa_message_type(*ctx, 0);
+    fko_set_spa_message(*ctx, NULL);
+    fko_set_spa_nat_access(*ctx, NULL);
+    fko_set_spa_server_auth(*ctx, NULL);
+    fko_set_spa_client_timeout(*ctx, 0);
+    fko_set_spa_digest_type(*ctx, 0);
+    fko_set_spa_digest(*ctx);
+    fko_set_spa_hmac_type(*ctx, 0);
+    fko_set_spa_encryption_mode(*ctx, 0);
+    fko_set_spa_data(*ctx, NULL);
+    fko_set_spa_hmac(*ctx, NULL, 0);
+    fko_set_raw_spa_digest_type(*ctx, 0);
+    fko_set_raw_spa_digest(*ctx);
+
+    spa_calls += 20;
+
+    return;
+}
+
+static void ctx_add_dupe_data(fko_ctx_t *ctx)
+{
+    fko_set_rand_value(*ctx, STR_8BYTES);
+    fko_set_rand_value(*ctx, STR_16BYTES);
+    fko_set_rand_value(*ctx, STR_16BYTES);
+    fko_set_username(*ctx, STR_64BYTES);
+    fko_set_username(*ctx, STR_64BYTES);
+    fko_set_timestamp(*ctx, 12345);
+    fko_set_timestamp(*ctx, 12345);
+    fko_set_spa_message_type(*ctx, FKO_ACCESS_MSG);
+    fko_set_spa_message_type(*ctx, FKO_ACCESS_MSG);
+    fko_set_spa_message(*ctx, SPA_MSG1);
+    fko_set_spa_message(*ctx, SPA_MSG1);
+    fko_set_spa_nat_access(*ctx, SPA_NAT_MSG);
+    fko_set_spa_nat_access(*ctx, SPA_NAT_MSG);
+    fko_set_spa_server_auth(*ctx, SERVER_AUTH_MSG);
+    fko_set_spa_server_auth(*ctx, SERVER_AUTH_MSG);
+    fko_set_spa_client_timeout(*ctx, 30);
+    fko_set_spa_client_timeout(*ctx, 30);
+    fko_set_spa_digest_type(*ctx, FKO_DEFAULT_DIGEST);
+    fko_set_spa_digest_type(*ctx, FKO_DEFAULT_DIGEST);
+    fko_set_spa_hmac_type(*ctx, FKO_DEFAULT_HMAC_MODE);
+    fko_set_spa_hmac_type(*ctx, FKO_DEFAULT_HMAC_MODE);
+    fko_set_spa_encryption_mode(*ctx, FKO_ENC_MODE_CBC);
+    fko_set_spa_encryption_mode(*ctx, FKO_ENC_MODE_CBC);
+    fko_set_spa_data(*ctx, STR_8BYTES);
+    fko_set_spa_data(*ctx, STR_64BYTES);
+
+    spa_calls += 24;
+
+    return;
+}
+
+
+static void
+test_loop_compounded(void)
+{
+    fko_ctx_t  ctx = NULL, decrypt_ctx = NULL;
+    char *spa_data = NULL;
+    int i, j, k, l, res;
+
+    for (i=0; i<FCN_CALLS; i++) {
+
+        fko_new(&ctx);
+
+        for (j=-1; j<FKO_LAST_MSG_TYPE+1; j++) {
+
+            res = fko_set_spa_message_type(ctx, j);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_message_type(): %s\n", fko_errstr(res));
+
+            res = fko_set_spa_client_timeout(ctx, i);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_client_timeout(): %s\n", fko_errstr(res));
+
+            res = fko_set_timestamp(ctx, 100);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_timestamp(): %s\n", fko_errstr(res));
+
+            fko_set_spa_message(ctx, SPA_MSG1);
+            res = fko_set_spa_message(ctx, SPA_MSG2);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_message(): %s\n", fko_errstr(res));
+
+            res = fko_set_spa_nat_access(ctx, SPA_NAT_MSG);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_nat_access(): %s\n", fko_errstr(res));
+
+            res = fko_set_username(ctx, "someuser");
+            if (res != FKO_SUCCESS)
+                printf("fko_set_username(): %s\n", fko_errstr(res));
+
+            res = fko_set_spa_server_auth(ctx, SERVER_AUTH_MSG);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_server_auth(): %s\n", fko_errstr(res));
+
+            res = fko_set_spa_hmac_type(ctx, FKO_HMAC_SHA256);
+            if (res != FKO_SUCCESS)
+                printf("fko_set_spa_hmac_type(): %s\n", fko_errstr(res));
+
+            for (k=-4; k<=16; k+=4) {
+                for (l=-4; l<=16; l+=4) {
+
+                    res = fko_spa_data_final(ctx, ENC_KEY, k, HMAC_KEY, l);
+                    if (res == FKO_SUCCESS) {
+                        res = fko_get_spa_data(ctx, &spa_data);
+                        if (res == FKO_SUCCESS) {
+
+                            res = fko_new_with_data(&decrypt_ctx, spa_data, NULL,
+                                0, FKO_ENC_MODE_CBC, HMAC_KEY, l, FKO_HMAC_SHA256);
+
+                            if (res == FKO_SUCCESS) {
+                                res = fko_decrypt_spa_data(decrypt_ctx, ENC_KEY, k);
+                                if (res != FKO_SUCCESS)
+                                    printf("fko_decrypt_spa_data(): %s\n", fko_errstr(res));
+
+                                fko_destroy(decrypt_ctx);
+                                decrypt_ctx = NULL;
+                                spa_calls += 13;
+                                spa_compounded_calls += 13;
+
+                            } else {
+                                printf("fko_new_with_data(): %s\n", fko_errstr(res));
+                            }
+                        } else {
+                            printf("fko_get_spa_data(): %s\n", fko_errstr(res));
+                        }
+                    } else {
+                        printf("fko_spa_data_final(): %s\n", fko_errstr(res));
+                    }
+                }
+            }
+        }
+        fko_destroy(ctx);
+        ctx = NULL;
+
+        spa_calls += 3;
+        spa_compounded_calls += 3;
+    }
+}
+
 static void
 test_loop(int new_ctx_flag, int destroy_ctx_flag)
 {
     fko_ctx_t  ctx = NULL, decrypt_ctx = NULL;
     int        i, j;
-    char       *spa_data = NULL;
+    char       *spa_data = NULL, encode_buf[100], decode_buf[100];
 
     printf("[+] test_loop(): %s, %s\n",
             new_ctx_flag == NEW_CTX ? "NEW_CTX" : "NO_NEW_CTX",
@@ -93,7 +398,7 @@ test_loop(int new_ctx_flag, int destroy_ctx_flag)
             NO_DIGEST, new_ctx_flag, destroy_ctx_flag);
 
     spa_func_int(&ctx, "fko_set_timestamp",
-            &fko_set_spa_client_timeout, -F_INT, F_INT, 10,
+            &fko_set_timestamp, -F_INT, F_INT, 10,
             new_ctx_flag, destroy_ctx_flag);
 
     for (i=0; i<FCN_CALLS; i++) {
@@ -224,20 +529,21 @@ test_loop(int new_ctx_flag, int destroy_ctx_flag)
         ctx_update(&ctx, new_ctx_flag, destroy_ctx_flag, DO_PRINT);
     }
 
-    /* NULL tests */
-    fko_set_rand_value(ctx, NULL);
-    fko_set_rand_value(ctx, NULL);
-    fko_set_username(ctx, NULL);
-    fko_set_username(ctx, NULL);
-    fko_set_spa_message(ctx, NULL);
-    fko_set_spa_message(ctx, NULL);
-    fko_set_spa_nat_access(ctx, NULL);
-    fko_set_spa_nat_access(ctx, NULL);
-    fko_set_spa_server_auth(ctx, NULL);
-    fko_set_spa_server_auth(ctx, NULL);
-    fko_set_spa_data(ctx, NULL);
-    fko_set_spa_data(ctx, NULL);
-    spa_calls += 12;
+    /* NULL tests
+    */
+    ctx_set_null_vals(&ctx);
+    ctx_set_null_vals(&ctx);
+
+    /* Now add data for code coverage - note data values
+     * are added twice in this function to exercise ctx
+     * member free() code.
+    */
+    ctx_add_dupe_data(&ctx);
+
+    /* set data values back to NULL
+    */
+    ctx_set_null_vals(&ctx);
+    ctx_set_null_vals(&ctx);
 
     for (i=0; i<FCN_CALLS; i++) {
         fko_destroy(ctx);
@@ -249,6 +555,23 @@ test_loop(int new_ctx_flag, int destroy_ctx_flag)
         decrypt_ctx = NULL;
     }
 
+    /* exercise the base64 encode/decode wrapper
+    */
+    fko_base64_encode((unsigned char *)ENC_KEY, encode_buf, 16);
+    fko_base64_decode(encode_buf, (unsigned char *)decode_buf);
+
+    /* call fko_errstr() across valid and invalid values
+    */
+    for (i=-5; i < FKO_LAST_ERROR+5; i++) {
+        printf("libfko error (%d): %s\n", i, fko_errstr(i));
+        spa_calls++;
+    }
+
+    printf("fko_new_with_data(): %s (data: %s)\n",
+        fko_errstr(fko_new_with_data(&decrypt_ctx, "tooshort", ENC_KEY,
+        strlen(ENC_KEY), FKO_ENC_MODE_CBC, HMAC_KEY, strlen(HMAC_KEY),
+        FKO_HMAC_SHA256)), "tooshort");
+
     return;
 }
 
@@ -497,4 +820,6 @@ display_ctx(fko_ctx_t ctx)
     printf(" Final SPA Data: %s\n", spa_data);
 
     spa_calls += 31;
+
+    return;
 }
